@@ -1,6 +1,6 @@
 /* Target-dependent code for the FT32.
 
-   Copyright (C) 2002-2020 Free Software Foundation, Inc.
+   Copyright (C) 2002-2024 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,13 +17,15 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#ifndef FT32_TDEP_H
-#define FT32_TDEP_H
+#ifndef GDB_FT32_TDEP_H
+#define GDB_FT32_TDEP_H
 
-struct gdbarch_tdep
+#include "gdbarch.h"
+
+struct ft32_gdbarch_tdep : gdbarch_tdep_base
 {
   /* Type for a pointer to a function.  Used for the type of PC.  */
-  struct type *pc_type;
+  struct type *pc_type = nullptr;
 };
 
-#endif /* FT32_TDEP_H */
+#endif /* GDB_FT32_TDEP_H */
